@@ -1,9 +1,9 @@
-from copy import deepcopy
 import os
 import sys
+from typing import Dict, Iterable, List
 
 import pluggy
-from tox.config import _split_env as split_env
+from tox.config import Config, _split_env as split_env
 from tox.reporter import verbosity1, verbosity2
 
 
@@ -12,17 +12,18 @@ hookimpl = pluggy.HookimplMarker("tox")
 
 @hookimpl
 def tox_configure(config):
+    # type: (Config) -> None
     verbosity2("original envconfigs: {}".format(list(config.envconfigs.keys())))
     verbosity2("original envlist: {}".format(config.envlist))
     verbosity2("original envlist_default: {}".format(config.envlist_default))
 
-    version = '.'.join([str(i) for i in sys.version_info[:2]])
+    version = get_python_version()
     verbosity2("Python version: {}".format(version))
 
     gh_actions_config = parse_config(config._cfg.sections.get("gh-actions", {}))
     verbosity2("tox-gh-actions config: {}".format(gh_actions_config))
 
-    factors = gh_actions_config["python"].get(version, [])
+    factors = get_factors(gh_actions_config, version)
     envlist = get_envlist_from_factors(config.envlist, factors)
     verbosity2("new envlist: {}".format(envlist))
 
@@ -34,15 +35,25 @@ def tox_configure(config):
 
 
 def parse_config(config):
+    # type: (Dict[str, str]) -> Dict[str, Dict[str, List[str]]]
     """Parse gh-actions section in tox.ini"""
-    config = deepcopy(config)
-    config["python"] = parse_dict(config.get("python", ""))
-    for k, v in config["python"].items():
-        config["python"][k] = split_env(v)
-    return config
+    config_python = parse_dict(config.get("python", ""))
+    # Example of split_env:
+    # "py{27,38}" => ["py27", "py38"]
+    return {
+        "python": {k: split_env(v) for k, v in config_python.items()}
+    }
+
+
+def get_factors(gh_actions_config, version):
+    # type: (Dict[str, Dict[str, List[str]]], str) -> List[str]
+    """Get a list of factors"""
+    return gh_actions_config["python"].get(version, [])
 
 
 def get_envlist_from_factors(envlist, factors):
+    # type: (Iterable[str], Iterable[str]) -> List[str]
+    """Filter envlist using factors"""
     result = []
     for env in envlist:
         for factor in factors:
@@ -53,12 +64,26 @@ def get_envlist_from_factors(envlist, factors):
     return result
 
 
+def get_python_version():
+    # type: () -> str
+    """Get Python version running in string (e.g,. 3.8)
+
+    - CPython => 2.7, 3.8, ...
+    - PyPy => pypy2, pypy3
+    """
+    if "PyPy" in sys.version:
+        return "pypy" + str(sys.version_info[0])
+    # Assuming running on CPython
+    return '.'.join([str(i) for i in sys.version_info[:2]])
+
+
 # The following function was copied from
 # https://github.com/tox-dev/tox-travis/blob/0.12/src/tox_travis/utils.py#L11-L32
 # which is licensed under MIT LICENSE
 # https://github.com/tox-dev/tox-travis/blob/0.12/LICENSE
 
 def parse_dict(value):
+    # type: (str) -> Dict[str, str]
     """Parse a dict value from the tox config.
     .. code-block: ini
         [travis]
